﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using IDTechSDK;

namespace WorldPayConsole
{

    




    internal class Program
    {
        static bool connected = false;
        static void Main(string[] args)
        {
            IDT_Device.setCallbackIP(MessageCallBack);
            Profile.discoverDevice();
            
            //wait for device connection
            while (!connected)
            {
                Common.pause(1000);
                Console.WriteLine("Waiting for device connection...");
            }


            bool stayInLoop = true;
            while (stayInLoop)
            {
                Console.WriteLine("Please enter selection");
                Console.WriteLine("1 Start Transaction");
                Console.WriteLine("2 Get Firmware");
                Console.WriteLine("3 Exit Application");
                ConsoleKeyInfo str1 = Console.ReadKey();
                char str = str1.KeyChar;
                if (str == '1')
                {
                    startTrans();
                }
                else if (str == '2')
                {
                    getfimrware();
                }
                else if (str == '3')
                {
                    stayInLoop = false;
                }
            }
            Console.WriteLine("Exiting Application");
        }



        static void showMessageOnLCD(byte[] language, byte[] message, string ident)
        {
            if (message == null) return;
            StringBuilder sb = new StringBuilder();
            int position = 0;
            while (position < message.Length)
            {
                byte line = message[position++];
                if (position < message.Length)
                {
                    if ((line & 0x80) == 0x80)
                    {
                        sb.Append(getLCDDisplayMessage(language, message[position++]));

                    }

                    position++;
                    if (position < message.Length)
                        sb.AppendLine();

                }


            }


            System.Diagnostics.Debug.WriteLine("Setting LCD Screen Message");
            output("\r\n" + sb.ToString() + "\r\n", ident);


        }

        static string getLCDDisplayMessage(byte[] language, int id)
        {
            if (id > (int)DisplayMessages.DISPLAY_MESSAGE_IDENTIFIER.TRANS_MSG_NEW_TBL_LIMIT) return "";
            ASCIIEncoding encoding = new ASCIIEncoding();
            string line1 = "";
            string line2 = "";
            string sLanguage = encoding.GetString(language, 0, 2).ToUpper();
            DisplayMessages.DISPLAY_MESSAGE_LANGUAGE theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.ENGLISH;
            if (sLanguage.Equals("FR"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.FRENCH;
            }
            if (sLanguage.Equals("EF"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.ENGLISH_FRENCH;
            }
            if (sLanguage.Equals("ES"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.SPANISH;
            }
            if (sLanguage.Equals("PT"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.PORTUGUESE;
            }
            if (sLanguage.Equals("ZH"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.CHINESE;
            }
            if (sLanguage.Equals("JA"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.JAPANESE;
            }

            DisplayMessages.retrieveMessage(theLang, (DisplayMessages.DISPLAY_MESSAGE_IDENTIFIER)id, ref line1, ref line2);
            return (line1 + "\r\n" + line2);
        }


        static string tlvToValues(byte[] tlv)
        {
            string text = "";
            Dictionary<string, string> dict = Common.processTLVUnencrypted(tlv);
            foreach (KeyValuePair<string, string> kvp in dict) text += kvp.Key + ": " + kvp.Value + "\r\n";
            dict.Clear();
            dict = null;
            return text;
        }
        static String GetTimestamp(string ident)
        {
            DateTime value = DateTime.Now;
            return (value.ToString("HH:mm:ss.fff") + "-" + ident + "  ");
        }
        static void output(string text, string ident)
        {

            Console.WriteLine(GetTimestamp(ident) + "\r\n" + text);
        }

        static void displayCardData(IDTTransactionData cardData, string ident)
        {
            string text = "";
            if (cardData != null && cardData.Base64 != null && cardData.Base64.Length > 0)
                text += "Raw Data (Base 64):\r\n" + cardData.Base64 + "\r\n\r\n";
            if (cardData != null && cardData.fastEMV != null && cardData.fastEMV.Length > 0)
                text += "Fast EMV Data: " + cardData.fastEMV + "\r\n\r\n";
            if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_PIN_DATA)
            {
                output("PIN Data received:\r\n", ident);
                if (cardData.pin_KSN != null && cardData.pin_KSN.Length > 0)
                {
                    output("KSN: " + cardData.pin_KSN + "\r\n", ident);

                }

                if (cardData.pin_pinblock != null && cardData.pin_pinblock.Length > 0)
                {
                    output("PINBLOCK: " + cardData.pin_pinblock + "\r\n", ident);

                }

                if (cardData.pin_KeyEntry != null && cardData.pin_KeyEntry.Length > 0)
                {
                    output("Key Entry: " + cardData.pin_KeyEntry + "\r\n", ident);

                }

                return;

            }

            if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_CARD_DATA)
            {
                if (cardData.msr_rawData != null)
                {

                    output("Raw Data (HEX): \r\n" + Common.getHexStringFromBytes(cardData.msr_rawData) + "\r\n", ident);

                }

            }

            if (cardData.captured_KSN != null)
            {
                //encrypted captured data
                if (cardData.captured_InitialVector != null)
                    text += "Man. Captured Initial Vector: " +
                            Common.getHexStringFromBytes(cardData.captured_InitialVector) + "\r\n";
                if (cardData.captured_KSN != null)
                    text += "Man. Captured KSN: " + Common.getHexStringFromBytes(cardData.captured_KSN) + "\r\n";
                if (cardData.captured_PAN != null)
                    text += "Man. Captured Encrypted PAN: " + Common.getHexStringFromBytes(cardData.captured_PAN) +
                            "\r\n";
                if (cardData.captured_Expiry != null)
                    text += "Man. Captured Encrypted Expiry: " +
                            Common.getHexStringFromBytes(cardData.captured_Expiry) + "\r\n";
                if (cardData.captured_CSC != null)
                    text += "Man. Captured Encrypted CSC: " + Common.getHexStringFromBytes(cardData.captured_CSC) +
                            "\r\n";
                if (cardData.captured_firstPANDigits != null && cardData.captured_firstPANDigits.Length > 0)
                    text += "PAN First Digits: " + cardData.captured_firstPANDigits + "\r\n";
                if (cardData.captured_lastPANDigits != null && cardData.captured_firstPANDigits.Length > 0)
                    text += "PAN Last Digits: " + cardData.captured_lastPANDigits + "\r\n";
                if (cardData.captured_MACValue != null)
                    text += "Man. Captured MAC: " + Common.getHexStringFromBytes(cardData.captured_MACValue) + "\r\n";
                if (cardData.captured_MACKSN != null)
                    text += "Man. Captured MAC KSN: " + Common.getHexStringFromBytes(cardData.captured_MACKSN) + "\r\n";




            }
            else if (cardData.captured_PAN != null)
            {
                //unecnrypted captured data
                if (cardData.captured_PAN != null)
                    text += "Man. Captured  PAN: " + System.Text.Encoding.UTF8.GetString(cardData.captured_PAN) +
                            "\r\n";
                if (cardData.captured_Expiry != null)
                    text += "Man. Captured  Expiry: " + System.Text.Encoding.UTF8.GetString(cardData.captured_Expiry) +
                            "\r\n";
                if (cardData.captured_CSC != null)
                    text += "Man. Captured  CSC: " + System.Text.Encoding.UTF8.GetString(cardData.captured_CSC) +
                            "\r\n";
            }



            if (cardData.device_RSN != null && cardData.device_RSN.Length > 0)
                text += "Serial Number: " + cardData.device_RSN + "\r\n";

            if (cardData.msr_track1Length > 0)
                text += "Track 1: " + cardData.msr_track1 + "\r\n";

            if (cardData.msr_encTrack1 != null)
                text += "Track 1 Encrypted: " + Common.getHexStringFromBytes(cardData.msr_encTrack1) + "\r\n";



            if (cardData.msr_hashTrack1 != null)
                text += "Track 1 Hash: " + Common.getHexStringFromBytes(cardData.msr_hashTrack1) + "\r\n";




            if (cardData.msr_track2Length > 0)
                text += "Track 2: " + cardData.msr_track2 + "\r\n";


            if (cardData.msr_encTrack2 != null)
                text += "Track 2 Encrypted: " + Common.getHexStringFromBytes(cardData.msr_encTrack2) + "\r\n";


            if (cardData.msr_hashTrack2 != null)
                text += "Track 2 Hash: " + Common.getHexStringFromBytes(cardData.msr_hashTrack2) + "\r\n";




            if (cardData.msr_track3Length > 0)
                text += "Track 3: " + cardData.msr_track3 + "\r\n";


            if (cardData.msr_encTrack3 != null)
                text += "Track 3 Encrypted: " + Common.getHexStringFromBytes(cardData.msr_encTrack3) + "\r\n";


            if (cardData.msr_hashTrack3 != null)
                text += "Track 3 Hash: " + Common.getHexStringFromBytes(cardData.msr_hashTrack3) + "\r\n";

            if (cardData.msr_KSN != null)
                text += "KSN: " + Common.getHexStringFromBytes(cardData.msr_KSN) + "\r\n";


            if (cardData.emv_clearingRecord != null)
            {
                if (cardData.emv_clearingRecord.Length > 0)
                {
                    text += "\r\nCTLS Clearing Record: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_clearingRecord) + "\r\n";
                    Dictionary<string, string> dict = Common.processTLVUnencrypted(cardData.emv_clearingRecord);
                    foreach (KeyValuePair<string, string> kvp in dict) text += kvp.Key + ": " + kvp.Value + "\r\n";
                    text += "\r\n\r\n";

                }

            }

            if (cardData.emv_unencryptedTags != null)
            {
                if (cardData.emv_unencryptedTags.Length > 0)
                {
                    text += "\r\n======================== \r\n";

                    text += "\r\nUnencrypted Tags: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_unencryptedTags) + "\r\n\r\n";
                    text += tlvToValues(cardData.emv_unencryptedTags);
                    text += "\r\n======================== \r\n";
                }

            }

            if (cardData.emv_encryptedTags != null)
            {
                if (cardData.emv_encryptedTags.Length > 0)
                {
                    text += "\r\n======================== \r\n";
                    text += "\r\nEncrypted Tags: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_encryptedTags) + "\r\n\r\n";
                    text += tlvToValues(cardData.emv_encryptedTags);
                    text += "\r\n======================== \r\n";

                }

            }

            if (cardData.emv_maskedTags != null)
            {
                if (cardData.emv_maskedTags.Length > 0)
                {
                    text += "\r\n======================== \r\n";
                    text += "\r\nMasked Tags: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_maskedTags) + "\r\n\r\n";
                    text += tlvToValues(cardData.emv_maskedTags);
                    text += "\r\n======================== \r\n";

                }

            }

            if (cardData.emv_hasAdvise) text += "CARD RESPONSE HAS ADVISE" + "\r\n";
            if (cardData.emv_hasReversal) text += "CARD RESPONSE HAS REVERSAL" + "\r\n";
            if (cardData.iccPresent == 1) text += "ICC Present: TRUE" + "\r\n";
            if (cardData.iccPresent == 2) text += "ICC Present: FALSE" + "\r\n";
            if (cardData.isCTLS == 1) text += "CTLS Capture: TRUE" + "\r\n";
            if (cardData.isCTLS == 2) text += "CTLS Capture: FALSE" + "\r\n";


            if (cardData.msr_extendedField != null && cardData.msr_extendedField.Length > 0)
                text += "Extended Field Bytes: " + Common.getHexStringFromBytes(cardData.msr_extendedField) + "\r\n";


            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TRANS_ARMOR_PKI)
                text += "Encryption Type: TRANS_ARMOR_PKI\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TRANS_ARMOR_TDES)
                text += "Encryption Type: TRANS_ARMOR_TDES\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_VOLTAGE)
                text += "Encryption Type: VOLTAGE\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_VISA_FPE)
                text += "Encryption Type: VISA_FPE\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_VERIFONE_FPE)
                text += "Encryption Type: VERIFONE_FPE\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_DESJARDIN)
                text += "Encryption Type: DESJARDIN\r\n";

            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TYPE_NONE)
                text += "Encryption Type: NONE\r\n";

            if (cardData.captureEncryptType != CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TYPE_NONE)
            {
                if (cardData.msr_keyVariantType == KEY_VARIANT_TYPE.KEY_VARIANT_TYPE_DATA)
                    text += "Key Type: Data Variant\r\n";
                else if (cardData.msr_keyVariantType == KEY_VARIANT_TYPE.KEY_VARIANT_TYPE_PIN)
                    text += "Key Type: PIN Variant\r\n";
            }



            if (cardData.mac != null)
                text += "MAC: " + Common.getHexStringFromBytes(cardData.mac) + "\r\n";

            if (cardData.macKSN != null)
                text += "MAC KSN: " + Common.getHexStringFromBytes(cardData.macKSN) + "\r\n";

            if (cardData.msr_sessionID != null)
                text += "Session ID: " + Common.getHexStringFromBytes(cardData.msr_sessionID) + "\r\n";

            if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_EMV_DATA)
            {
                switch (cardData.emv_resultCode)
                {
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_APPROVED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_APPROVED" + "\r\n");
                        output("APPROVED", ident);
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_APPROVED_OFFLINE:
                        output("APPROVED", ident);
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_APPROVED_OFFLINE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_DECLINED_OFFLINE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_DECLINED_OFFLINE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_DECLINED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_DECLINED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_GO_ONLINE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_GO_ONLINE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CALL_YOUR_BANK:
                        output("CALL YOUR BANK", ident);
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CALL_YOUR_BANK" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_NOT_ACCEPTED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_NOT_ACCEPTED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_FALLBACK_TO_MSR:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_FALLBACK_TO_MSR" + "\r\n");
                        output("", ident);
                        output(getLCDDisplayMessage(new byte[] { 0x45, 0x4E }, (int)DisplayMessages.DISPLAY_MESSAGE_IDENTIFIER.MSG_NEW_WELCOME), ident);
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_TIMEOUT:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_TIMEOUT" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_AUTHENTICATE_TRANSACTION:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_AUTHENTICATE_TRANSACTION" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_SWIPE_NON_ICC:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_SWIPE_NON_ICC" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CTLS_TWO_CARDS:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CTLS_TWO_CARDS" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CTLS_TERMINATE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CTLS_TERMINATE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CTLS_TERMINATE_TRY_ANOTHER:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CTLS_TERMINATE_TRY_ANOTHER" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_GO_ONLINE_CTLS:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_GO_ONLINE_CTLS" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_FAILED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_FAILED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_MSR_SWIPE_CAPTURED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_MSR_SWIPE_CAPTURED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_REQUEST_ONLINE_PIN:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_REQUEST_ONLINE_PIN" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_REQUEST_SIGNATURE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_REQUEST_SIGNATURE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_ADVISE_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_ADVISE_REQUIRED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_REVERSAL_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_REVERSAL_REQUIRED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_ADVISE_REVERSAL_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_ADVISE_REVERSAL_REQUIRED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_NO_ADVISE_REVERSAL_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_NO_ADVISE_REVERSAL_REQUIRED" + "\r\n");
                        break;
                    default:
                        text += ("EMV RESULT: " + errorCode.getErrorString((RETURN_CODE)cardData.emv_resultCode) +
                                 "\r\n");
                        break;
                }
            }

            if (cardData.emv_transaction_Error_Code > 0)
                text += ("Transaction Error: " + errorCode.getTransError(cardData.emv_transaction_Error_Code) + "\r\n");
            if (cardData.emv_RF_State > 0)
                text += ("RF State: " + errorCode.getRFState(cardData.emv_RF_State) + "\r\n");
            if (cardData.emv_ESC > 0)
                text += ("Extended Status Code: " + errorCode.getExtendedStatusCode(cardData.emv_ESC) + "\r\n");
            if (cardData.emv_appErrorFn > 0)
                text += ("App Error Function: " + errorCode.getEMVAppErrorFn(cardData.emv_appErrorFn) + "\r\n");
            if (cardData.emv_appErrorState > 0)
                text += ("App Error State: " + errorCode.getEMVAppErrorState(cardData.emv_appErrorState) + "\r\n");



            if (cardData.ctlsApplication > 0)
            {
                text += "Contactless Application: ";
                switch (cardData.ctlsApplication)
                {
                    case CTLS_APPLICATION.CTLS_APPLICATION_AMEX:
                        text += ("CTLS_APPLICATION_AMEX" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DISCOVER:
                        text += ("CTLS_APPLICATION_DISCOVER" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_MASTERCARD:
                        text += ("CTLS_APPLICATION_MASTERCARD" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_VISA:
                        text += ("CTLS_APPLICATION_VISA" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_SPEEDPASS:
                        text += ("CTLS_APPLICATION_SPEEDPASS" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_GIFT_CARD:
                        text += ("CTLS_APPLICATION_GIFT_CARD" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DINERS_CLUB:
                        text += ("CTLS_APPLICATION_DINERS_CLUB" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_EN_ROUTE:
                        text += ("CTLS_APPLICATION_EN_ROUTE" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_JCB:
                        text += ("CTLS_APPLICATION_JCB" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_VIVO_DIAGNOSTIC:
                        text += ("CTLS_APPLICATION_VIVO_DIAGNOSTIC" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_HID:
                        text += ("CTLS_APPLICATION_HID" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_MSR_SWIPE:
                        text += ("CTLS_APPLICATION_MSR_SWIPE" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_RESERVED:
                        text += ("CTLS_APPLICATION_RESERVED" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DES_FIRE_TRACK_DATA:
                        text += ("CTLS_APPLICATION_DES_FIRE_TRACK_DATA" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DES_FIRE_RAW_DATA:
                        text += ("CTLS_APPLICATION_DES_FIRE_RAW_DATA" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_RBS:
                        text += ("CTLS_APPLICATION_RBS" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_VIVO_COMM:
                        text += ("CTLS_APPLICATION_VIVO_COMM" + "\r\n");
                        break;

                }
            }



            output(text, ident);

            text = "";
            byte[] temp = new byte[0];




        }


        static void MessageCallBack(IDTechSDK.IDT_DEVICE_Types type, DeviceState state, byte[] data,
 IDTTransactionData cardData, EMV_Callback emvCallback, RETURN_CODE transactionResultCode, string ident)
        {


            IDTechComm comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_USB, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_SERIAL, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_BLE, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_AUDIO_JACK, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_IP, ident);
            DEVICE_INTERFACE_Types connect = DEVICE_INTERFACE_Types.DEVICE_INTERFACE_UNKNOWN;
            if (comm != null)
            {
                connect = comm.getDeviceConnection();
            }


            switch (state)
            {


                case DeviceState.DefaultDeviceTypeChange:
                    output("\nSDK Default Device =  " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect),ident);



                    break;
                case DeviceState.Connected:
                    Console.WriteLine("Connected " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect, ident));
                    connected = true;
                    break;
                case DeviceState.Disconnected:
                    Console.WriteLine("Disconnected " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect, ident));
                    connected = false;
                    break;
                case DeviceState.ConnectionFailed:
                    break;
                case DeviceState.RKI:
                    break;

                case DeviceState.SocketEstablished:
                    break;
                case DeviceState.SocketFailed:
                    break;
                case DeviceState.ViVOconfig:
                    break;
                case DeviceState.TransactionData:


                    if (cardData == null)
                    {
                        output("Card Data Null - Breaking", ident);
                        break;
                    }

                    output("cardData.emv_resultCode = " + cardData.emv_resultCode.ToString(), ident);



                    if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_PIN_DATA ||
                        cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_CARD_DATA ||
                        cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_EMV_DATA)

                    {

                        displayCardData(cardData, ident);
                    }

                    break;

                case DeviceState.EMVCallback:


                    if (emvCallback == null) break;
                    if (emvCallback.callbackType == EMV_CALLBACK_TYPE.EMV_CALLBACK_TYPE_LCD) //LCD Callback Type
                    {
                        if (emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_CLEAR_SCREEN)
                        {

                            return;
                        }
                        else if (emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_MESSAGE || emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE2 || emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE3)
                        {
                            showMessageOnLCD(emvCallback.language, emvCallback.lcd_messages, ident);
                        }
                        else if (emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE4)
                        {
                            showMessageOnLCD(emvCallback.language, emvCallback.lcd_messages, ident);

                            output("Mode 4 Callback - Awaiting Response.  \r\n", ident);
                            RETURN_CODE RT = IDT_Device.SharedController.emv_callbackResponseLCD(EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE4, 0);

                        }


                    }



                    break;
            }

        }


        static WorldPay wp = null;
        static void startTrans()
        {
            if (wp == null) wp = new WorldPay();

            WorldPay.WorldPayData data = new WorldPay.WorldPayData();

            data.accountID = "1188346";
            data.accountToken = "E1EB3EFB049DFB599F1CB454E1CFC4FD14BF90BCE74456AE9E9490D7D609B466C81A3801";
            data.acceptorID = "364798674";
            data.transactionType = WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_SALE;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_PRE_AUTH;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_COMPLETION;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_VOID;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_RETURN;   //specify Sale, Pre-Auth, or Completion
            data.amount = "32.00";   //specify transaction Amount
            data.timeout = 30;  //Timeout for transaction
            data.isTest = true;  //if test transaction, set to TRUE.  If live transactions, set to FALSE
            data.msrOnly = false; //for EMV + MSR, set to FALSE.  If MSR swipe only, set to TRUE
            data.transactionID = ""; //required if COMPLETION, VOID or RETURN.  Otherwise, leave blank
            data.referenceNumber = ""; //required value.  If blank, SDK will put in random number
            data.terminalID = "12345"; //required value.  Set a terminal identifer
            data.ticketNumber = ""; //required value.  If blank, SDK will put in random number
            data.duplicateCheck = false;  //set to TRUE if you want to check for duplicate transactions
            data.duplicateOverride = true;  //set to TRUE if you want to allow duplicate charges to same card
            data.enableCTLS = true;  //set to TRUE to allow Contactless payments on VP3300/VP8300
            data.deviceID = "";  //set to device identifier to target specific device, or leave empty to target first/only device.
            wp.returnRequest = true; //set to TRUE if you would like the request packet returned with the response

            bool requestOnly = false; // If this is TRUE, the SDK will return the request packet instead of contacting WorldPay with it. For integrations that want to control communication with WorldPay servers

            string ident = data.deviceID; // If you want to target a specific device, you specify either as a parameter you pass to executeTransaction, or through data.deviceID. Either method is acceptable 
            WorldPay.bypassNetworkChecking = true; //if you don't want to slow down the transaction by checking for network connectivity first, set this to true
            wpResponse = false;
            RETURN_CODE rt = wp.executeTransaction(data, WPCallback, requestOnly, ident);

            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                output("Start Transaction Successful\r\n", IDT_Device.currentIdent());
                Console.WriteLine("Waiting for WorldPay Response");
                while (!wpResponse) Common.pause(1000);   //wait until WPCallback receives a response
            }
            else
            {
                output("Start transaction failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) +
                                    ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n", IDT_Device.currentIdent());

            }

        }

        static bool wpResponse = false;
        static void WPCallback(IDT_DEVICE_Types sender, string xmlResponse, RETURN_CODE resultCode, EMV_RESULT_CODE emvCode, string ident, IDTTransactionData cardData, WorldPay.ReceiptData receipt)
        {

            if (cardData != null)
            {
                output("\r\n\r\n", ident);
                displayCardData(cardData, ident);
            }
            output("\r\n\r\nResponse received from Worldpay: " + resultCode.ToString() + "\r\n\r\n", ident);
            output(xmlResponse, ident);
            if (cardData != null)
            {
                output("\r\n\r\n", ident);
                output(getReceipt(receipt), ident);
            }
            wpResponse = true;
        }

        static void cancelTrans()
        {
            if (wp == null) return;
            RETURN_CODE rt = wp.cancelTransaction();

            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                output("Cancel Transaction Successful\r\n", IDT_Device.currentIdent());
            }
            else
            {
                output("Cancel transaction failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) +
                                    ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n", IDT_Device.currentIdent());
            }
        }

        static void getfimrware()
        {
            string fwver = "";
            RETURN_CODE rt = IDT_Device.SharedController.device_getFirmwareVersion(ref fwver);

            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                output("Firmware Version: " + fwver, IDT_Device.currentIdent());
            }
            else
            {
                output("Firmware version failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) +
                                    ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n", IDT_Device.currentIdent());
            }
        }

        static string getReceipt(WorldPay.ReceiptData receipt)
        {
            String rec = "";
            rec = rec + "================================";
            rec = rec + printCenter("\r\n");
            rec = rec + printCenter("RECEIPT");
            rec = rec + printCenter("\r\n");

            rec = rec + "================================";
            rec = rec + printCenter("\r\n");
            string dateString = receipt.dateTime.ToString("yyyy/MM/dd HHmmss");
            rec = rec + printCenter("Joe's Resaturant");
            rec = rec + printCenter("\r\n");

            rec = rec + printCenter("10721 Walker Street");
            rec = rec + printCenter("\r\n");
            rec = rec + printCenter("Cypress, CA 90630");
            rec = rec + printCenter("\r\n");
            rec = rec + printCenter("714-761-8886");
            rec = rec + printCenter("\r\n");
            rec = rec + printCenter("www.joesrestaurant.net");
            rec = rec + printCenter("\r\n");



            rec = rec + "--------------------------------";
            rec = rec + "\r\n";
            switch (receipt.saleType)
            {
                case WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_SALE:
                    rec = rec + printCenter("SALE");
                    break;
                case WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_PRE_AUTH:
                    rec = rec + printCenter("PRE-AUTH");
                    break;
                case WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_COMPLETION:
                    rec = rec + printCenter("COMPLETION");
                    break;
                case WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_VOID:
                    rec = rec + printCenter("VOID");
                    break;
                case WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_RETURN:
                    rec = rec + printCenter("RETURN");
                    break;
                case WorldPay.TRANSACTION_TYPE.TRANSACTION_TYPE_CREDIT:
                    rec = rec + printCenter("CREDIT");
                    break;
                default:
                    break;
            }
            rec = rec + "\r\n";
            if (!receipt.isApproved)
            {
                rec = rec + printCenter("TRANSACTION FAILED");
            }
            else
            {
                rec = rec + printCenter("TRANSACTION SUCCESSFUL");
            }
            rec = rec + "\r\n";
            rec = rec + printCenter(receipt.result);

            rec = rec + "\r\n";

            if (receipt.entryMethod == WorldPay.ENTRY_METHOD.ENTRY_METHOD_SWIPE) rec = rec + printJustified("Entry Method: ", "SWIPE");
            if (receipt.entryMethod == WorldPay.ENTRY_METHOD.ENTRY_METHOD_CTLS) rec = rec + printJustified("Entry Method: ", "CTLS");
            if (receipt.entryMethod == WorldPay.ENTRY_METHOD.ENTRY_METHOD_CONTACT) rec = rec + printJustified("Entry Method: ", "CONTACT");

            rec = rec + "\r\n";
            rec = rec + "--------------------------------";
            rec = rec + "\r\n";
            if (!String.IsNullOrEmpty(receipt.card))
            {
                rec = rec + printJustified("Card:", receipt.card) + "\r\n";
            }
            if (!String.IsNullOrEmpty(receipt.account))
            {
                rec = rec + printJustified("Account:", receipt.account) + "\r\n";
            }

            rec = rec + printJustified("Transaction Type:", receipt.transType);
            rec = rec + "\r\n";
            rec = rec + printJustified("Date/Time:", dateString);

            rec = rec + "\r\n";

            rec = rec + printJustified("Amount:", receipt.amount);
            rec = rec + "\r\n";
            if (!String.IsNullOrEmpty(receipt.iad))
            {
                rec = rec + printJustified("IAD:", receipt.iad);
                rec = rec + "\r\n";
            }
            if (!String.IsNullOrEmpty(receipt.aid))
            {
                rec = rec + printJustified("AID:", receipt.aid);
                rec = rec + "\r\n";
            }
            if (!String.IsNullOrEmpty(receipt.authCode))
            {
                rec = rec + printJustified("AUTH:", receipt.authCode);
                rec = rec + "\r\n";
            }
            if (!String.IsNullOrEmpty(receipt.transID))
            {
                rec = rec + printJustified("TRANS ID:", receipt.transID);
                rec = rec + "\r\n";
            }
            if (!String.IsNullOrEmpty(receipt.cert))
            {
                rec = rec + printJustified("TC:", receipt.cert);
                rec = rec + "\r\n";
            }

            if (receipt.cvm == WorldPay.CVM_TYPE.CVM_TYPE_NO_CVM)
            {
                rec = rec + "\r\n\r\nNO CVM\r\n\r\n";

            }
            if (receipt.cvm == WorldPay.CVM_TYPE.CVM_TYPE_SIGNATURE)
            { 
                    rec = rec + "Card Holder Signature:\r\n\r\n\r\n\r\n------------------------------\r\n" + receipt.name + "\r\n\r\nI Acknowledge Satisfactory Rec-\r\n";
                    rec = rec + "eipt Of Relative Goods/Services\r\n\r\n";
            }

            rec = rec + "\r\n\r\n";
            return rec;

        }

        static string printCenter(String str)
        {
            if (str == null) str = "";
            int len = str.Length;
            if (len > 30)
            {
                return str;
            }
            for (int x = 0; x < (32 - len) / 2; x++)
            {
                str = " " + str;
            }
            return str;
        }


        static string printJustified(String str, String str2)
        {
            if (str == null) str = "";
            if (str2 == null) str2 = "";
            int len = str.Length + str2.Length;
            if (len > 31)
            {
                return str + str2;
            }
            for (int x = len; x < 32; x++)
            {
                str2 = " " + str2;
            }
            return str + str2;
        }



    }
}
